#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

require "http.pl";
require "html.pl";
require "db.pl";
require "dbops.pl";
require "session.pl";
require "url.pl";
require "urllist.pl";
require "common.pl";


# 0: latest url id
# 1: count
# 2: date format
# ...: ignored chans
# r: array of hash (url_id, url_url, url_count, url_date, chan_id, chan_name,
#			nick_id, nick_name)
sub get_latest($$@)
{
	my $latest_id = shift;
	my $count = shift;
	my $date_fmt = shift;
	my @ignored = @_;

	my $ignore;
	my ($query, $result);
	my @ret;

	$ignore = "";
	if ($#ignored >= 0) {
		foreach my $id (@ignored) {
			$ignore .= " AND urls.chan != $_[0]";
		}
		$ignore =~ s/ AND (.*)/AND ($1) /;
	}

	$count = &Common::min($count, 100);

	my $to_char = &DBOps::get_date_fmt("urls.time", "time", $date_fmt);
	$query = "SELECT urls.id, urls.url, urls.count, $to_char, " .
				"chans.id, chans.chan, " .
				"nicks.id, nicks.nick " .
			"FROM urls, chans, nicks " .
			"WHERE urls.id > $latest_id $ignore " .
				"AND urls.nick = nicks.id " .
				"AND urls.chan = chans.id " .
			"ORDER BY urls.id ASC " .
			"LIMIT $count";
	$result = &DB::query($query);
	($result->ntuples == 0) && return ();

	while (my ($url_id, $url_url, $url_count, $url_date,
				$chan_id, $chan_name,
				$nick_id, $nick_name) = $result->fetchrow) {
		my %t = (
			'url_id' => $url_id,
			'url_url' => $url_url,
			'url_date' => $url_date,
			'url_count' => $url_count,
			'chan_id' => $chan_id,
			'chan_name' => $chan_name,
			'nick_id' => $nick_id,
			'nick_name' => $nick_name
			);
		push(@ret, \%t);
	}
	return @ret;
}


# 0: sid
sub latest($)
{
	my ($sid) = @_;

	my %data = &Session::get_cookie_data($sid);
	my @ignore = &Session::get_ignore_chans($sid);
	my @urls = &get_latest($data{'latest'}, $data{'urls'},
			$data{'latest_date'}, @ignore);

	if ($data{'reset'} == 1) {
		&Session::set_latest($sid, &Session::get_latest_url_id());
	}

	if ($#urls == -1) {
		printf "<h1>%s</h1>\n", &Lang::get("latest-no-new-urls");
		return;
	}

	printf "<h1>%s</h1>\n", &Lang::get("latest-label");
	&URLList::print(\@urls, "url", "nick", "chan", "date", "count");

	if ($data{'reset'} == 0) {
		printf "<p><a href=\"%s\">%s</a></p>\n",
				&URL::latest(1),
				&Lang::get("latest-reset");
	}
}


sub main()
{
	my $sid;
	my $phase;

	$sid = &Session::get_session();
	&Session::check_sid($sid, &URL::session(1, &URL::latest()));

	if (&HTTP::get("reset", 0) == 1) {
		&Session::set_latest($sid, &Session::get_latest_url_id());
		&HTTP::redirect(&URL::latest());
		&HTTP::print_header();
		exit(0);
	}

	&HTTP::print_header();
	&HTML::print_header();

	&Session::print_links();

	&latest($sid);

	&HTML::print_footer();
}


&main();
